from datetime import datetime, timedelta
import aiosqlite
import sqlite3
from typing import List, Dict
from abc import ABC, abstractmethod
from collections import defaultdict
from typing import List, Dict
from asyncio import Lock



# 核心接口
class IHistoryManager(ABC):
    
    @abstractmethod
    async def save_message(self, session_id: str, role: str, content: str) -> None:
        pass

    @abstractmethod
    async def get_history(
        self, 
        session_id: str
    ) -> List[Dict]:
        pass
    
    @abstractmethod
    async def clear_history(self, session_id: str) -> None:
        pass
    



class SQLiteHistoryManager(IHistoryManager):
    
    def __init__(self, db_path: str,):
        self.write_lock = Lock()
        self.db_path = db_path
        self.init_db()

    def init_db(self):
        with sqlite3.connect(self.db_path) as db:
            db.execute('''
                CREATE TABLE IF NOT EXISTS history (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    session_id TEXT,
                    role TEXT,
                    content TEXT,
                    timestamp DATETIME DEFAULT CURRENT_TIMESTAMP
                )
            ''')
            db.commit()


    async def save_message(self, session_id: str, role: str, content: str) -> None:

        async with self.write_lock:
            async with aiosqlite.connect(self.db_path) as db:
                await db.execute(
                    "INSERT INTO history (session_id, role, content) VALUES (?, ?, ?)",
                    (session_id, role, content)
                )
                await db.commit()


    async def get_history(self, 
        session_id: str, length: int, seconds: int
    ) -> List[Dict[str, str]]:
        
        if not length:
            return []

        async with aiosqlite.connect(self.db_path) as db:
            cursor = await db.execute(
                "SELECT role, content FROM history WHERE "
                "session_id = ? AND timestamp >= datetime('now', ?)"
                "ORDER BY timestamp DESC LIMIT ?",
                (session_id,f'-{seconds} seconds',length,))
            rows = await cursor.fetchall()
        return [{"role": row[0], "content": row[1]} for row in reversed(rows)]
    
    
    
    async def clear_history(self, session_id: str) -> None:
        async with aiosqlite.connect(self.db_path) as db:
            await db.execute("DELETE FROM history WHERE session_id = ?", (session_id,))
            await db.commit()

__all__ = [
    "IHistoryManager",
    "SQLiteHistoryManager", 
]


from .llm_service import LLMService
from .message_handle import *
from .config import Config, LLMParams
from nonebot.plugin import PluginMetadata


__plugin_meta__ = PluginMetadata(
    name="nonebot-plugin-anywhere-llm",
    description="为插件提供 LLM 访问能力， 统一记忆管理，支持多种模型参数和自定义 Prompt",
    type="library",
    usage='创建 llm_service 在插件内使用',
    homepage="https://github.com/Zeta-qixi/nonebot-plugin-anywhere-llm",
    config=Config,
    supported_adapters={"~onebot.v11"},
)





from abc import ABC, abstractmethod
import re
import os
from pathlib import Path
from string import Template
from typing import Dict, Union
from nonebot import logger

from ..config import SystemPromptConfig, AVATAT_DIR, TEMPLATE_DIR

class DefaultDict(dict):
    def __getitem__(self, key):
        try:
            return super().__getitem__(key)
        except KeyError:
            logger.warning(f"Key '{key}' is missing, replaced with empty string")
            return ''
        
        
        
class PromptTemplate(ABC):
    """Prompt模板基类"""
    _PATH_PATTERN = re.compile(r'^Path\((.*)\)$', re.IGNORECASE)
    

    def _resolve_value(self, value: Union[str, Path]) -> str:
        """统一解析值：处理文件路径或直接返回值"""
        if isinstance(value, (str, Path)):
            str_value = str(value).strip()
            match = self._PATH_PATTERN.match(str_value)
            if match:
                file_path = match.group(1).strip('\'"')
                try:
                    with open(file_path, 'r', encoding='utf-8') as f:
                        return f.read()
                except Exception as e:
                    raise ValueError(f"文件读取失败 [{file_path}]: {str(e)}")
            return str_value
        return value
    
    @abstractmethod
    def render(self) -> Dict[str, str]:
        pass
    
    @staticmethod
    def load_template(template: str, dir: Path) -> str:
        """加载模板内容，处理文件路径或直接返回模板字符串"""
        if len(template) <= 20:
            for ext in ['.md', '.txt', '']:
                for file in [dir/(template+ext), Path(template+ext)]:
                    try:
                        if file.is_file():
                            logger.info(f'加载模板成功: {file}')
                            return file.read_text( encoding="utf-8")
                    except Exception as e:
                        ...
        return template


class SystemPromptTemplate(PromptTemplate):
    """系统提示模板"""
    def __init__(self, config: SystemPromptConfig):
        
        self.context = DefaultDict()
        for k, v in config.model_dump().items():
            if k == 'template':
                self.template = Template(self.load_template(v, TEMPLATE_DIR))
            else:
                self.context[k] = self.load_template(v, AVATAT_DIR)
        
            
    def set_context(self, context: Dict) -> None:
        self.context.update(context)

    def render(self) -> Dict[str, str]:
        content = self.template.substitute(self.context)
        return {"role": 'system', "content": content}
    

class UserPromptTemplate(PromptTemplate):
    """用户动态模板 主要是用户身份 用与群聊"""
    def __init__(self, role: str = "user"):
        self.role = role

    def set_role_list(self, context: Dict) -> None:
        ## TODO
        self.context = context
        ## 默认 通过 config（master 默认称呼..） 或 动态 设置 user 的身份
        ## 先试试不修改system prompt 有没有效果

    def render(self, user_id: str = None) -> Dict[str, str]:
        
        if user_id:
            ...
        content = self.context.get("user_input", "")
        return {"role": self.role, "content": str(content)}
    




from typing import Dict, List, Callable
from .history_manager import SQLiteHistoryManager
from .prompt_templates import  SystemPromptTemplate
from .injectors import InformationInjector, create_time_injector, create_weather_injector
from ..config import MessagesConfig, PromptInjectionConfig


class MessageHandler:
    def __init__(self, config: MessagesConfig):
        
        self.history_mgr = SQLiteHistoryManager(
            db_path=config.history.db_path)
        self.system_prompt = SystemPromptTemplate(
            config.system_prompt)
        self.injector = InformationInjector()
        self._setup_injectors(config.injections)
        
        self.histroy_length=config.history.max_length
        self.histroy_time=config.history.time_window


    def _setup_injectors(self, config: PromptInjectionConfig):
        """配置信息注入器
        
        Args:
            time_level: 时间信息等级
                0: 不注入
                1: 基础时间 2: 日期+季节 3: 节日信息（待实现）
            weather_level: 天气信息等级
                0: 不注入
                1: 基础天气
        """
        if config.time:
            self.injector.register_injector(
                'time', 
                create_time_injector(config.time))
        if config.weather:
            self.injector.register_injector(
                'weather', 
                create_weather_injector(config.weather))


    def add_injector(self, func: Callable[[str], str], priority):
        self.injector.register_injector(func, priority)
    
    async def save_message(self, session_id: str, data: Dict[str, str]) -> None:
        await self.history_mgr.save_message(session_id, data.get('role'),  data.get('content'))
        
        
    async def process_message(
        self,
        session_id: str,
        user_input: str,
    ) -> List[Dict[str, str]]:
        
        self.injector.inject(self.system_prompt) 
        system_prompt = self.system_prompt.render()
        histroy = await self.history_mgr.get_history(session_id, self.histroy_length, self.histroy_time)
        user_input = {'role': 'user', 'content': user_input}
        messages = [system_prompt] + histroy + [user_input]
        
        return messages


from pathlib import Path
from ..config import DATA_DIR


def load_template( name: str, template_dir: str = DATA_DIR) -> str:
    """
    从模板目录加载模板文件，自动匹配 .md、.txt 或无扩展名文件
    
    """
    dir = Path(template_dir)
    for ext in ['.md', '.txt', '']:
        path = dir / f"{name}{ext}"
        if path.exists():
            with open(path, 'r', encoding='utf-8') as f:
                return f.read().strip()
    raise FileNotFoundError(f"未找到文件: {name}")


import datetime
from typing import Any, Callable, Dict, Tuple
from datetime import datetime
from collections import OrderedDict
from ..prompt_templates import SystemPromptTemplate



class InformationInjector:
    def __init__(self):

        self.injectors: Dict[str, Tuple[int, Callable[[Any], str]]] = OrderedDict()
    
    def register_injector(self, name: str, 
                         injector: Callable[[Any], str],
                         priority: int = 0,) -> None:
        
        self.injectors[name] = (priority, injector)
        
    def inject(self, system_prompt: SystemPromptTemplate) -> None:
        for name, (_, injector) in sorted(self.injectors.items(), key=lambda x: x[1][0]):
            system_prompt.set_context({name: injector()})
            


# ================================ #        
#           内置注入               
# ================================ #     
   
   
SEASON = [0, '冬', '冬', '春', '春', '春', '夏', '夏', '夏', '秋', '秋', '秋', '冬']
def llm_system_time() -> str:
    now = datetime.now()
    formatted_date = now.strftime("%D %H:%M %A ") + SEASON[now.month]
    return f"[{formatted_date}]" 


def create_time_injector(option: int) -> Callable[[str], str]:
    def time_injector() -> str:
        
        now = datetime.now()
        if option == 1:
            context = now.strftime("%H:%M")
        elif option == 2:
            context = now.strftime("%D %H:%M %A ") + SEASON[now.month]
        elif option == 3:
            # TODO add holiday
            context = now.strftime("%D %H:%M %A ") + SEASON[now.month]
        return context   
    return time_injector

def get_weather():
    ...
    

def create_weather_injector(option: int) -> Callable[[str], str]:
    def weather_injector() -> str:
        return get_weather()
    
    return weather_injector

__all__=["create_time_injector", "create_weather_injector"]


import yaml
import random
from pathlib import Path
from typing import Dict, Any
from nonebot.adapters.onebot.v11 import MessageEvent, GroupMessageEvent

from .message_handle import MessageHandler, PromptTemplate
from .config import AppConfig, Config_DIR, deep_update_model
from .provider import openai_provider



def _flatten_dict(data: Dict, parent_key: str = '') -> list:
    """字典扁平化处理"""
    items = []
    for k, v in data.items():
        new_key = f"{parent_key}.{k}" if parent_key else k
        if isinstance(v, dict):
            items.extend(_flatten_dict(v, new_key))
        else:
            items.append((new_key, v))
    return items


# LLMService.py
class LLMService:
    def __init__(self,config: AppConfig = None):
        self.config = config or AppConfig()     

    @classmethod
    def load(cls, file_name: str | Path) -> 'LLMService':
        file_path = Config_DIR / file_name
        with open(file_path) as f:
            data = yaml.safe_load(f)
        config = deep_update_model(AppConfig(), data)
        return cls(config = config)



        
    def to_dict(self) -> Dict:
        """导出为字典"""
        return self.config.model_dump()

    def save(self, file_name: str | Path) -> None:
        data = self.to_dict()
        del(data['params']['api_key'])
        file_path = Config_DIR / file_name
        with open(file_path, 'w', encoding='utf-8') as f:
            yaml.safe_dump(
                data,
                f,
                allow_unicode=True,
                sort_keys=False
            )

        
    async def generate(
            self,
            input: str,
            event: MessageEvent = None,
            save: bool = False,
        ) -> str:
            
            message_handle = MessageHandler(self.config.messages)
            session_id = event.get_session_id() if event else 'default'
            messages = await message_handle.process_message(session_id, input)
            response = await openai_provider.generate(
                messages = messages,
                params = self.config.params
            )
            if save:
                await message_handle.save_message(session_id, messages[-1])
                await message_handle.save_message(session_id, {'role': 'assistant' ,'content': response})
            return response



from abc import ABC, abstractmethod
from typing import Any, List, Tuple

class BaseLLMProvider(ABC):
    @abstractmethod
    async def generate(self, messages: List[Tuple[str, str]], **kwargs: Any) -> str:
        pass


# from typing import Dict
# from nonebot import require
# from nonebot.matcher import Matcher
# from nonebot import get_driver, on_command, on_message
# from nonebot.adapters.onebot.v11 import GroupMessageEvent, MessageSegment



# from .. import LLMParams, LLMService

# my_params = LLMParams(
#     model= "deepseek-ai/DeepSeek-R1-Distill-Qwen-7B",

# )
# llm = LLMService(my_params)

# test_matcher = on_command("test")
# @test_matcher.handle()
# async def handle_ask(matcher: Matcher, event: GroupMessageEvent):  
#     res = await llm.generate('回复测试')
#     await matcher.finish(res)
    




from pydantic import BaseModel, Field
from nonebot import get_plugin_config
from nonebot import require

from pydantic import BaseModel, Field
from typing import Any, Literal, Dict

require("nonebot_plugin_localstore")
import nonebot_plugin_localstore as store

DATA_DIR = store.get_plugin_data_dir()
Config_DIR = store.get_plugin_config_dir()
AVATAT_DIR = DATA_DIR / 'avatar'
TEMPLATE_DIR = DATA_DIR /'template'
DB_PATH = store.get_plugin_data_file("history.db")
for path in [AVATAT_DIR, TEMPLATE_DIR]:
    path.mkdir(parents=True, exist_ok=True)

class Config(BaseModel):

    openai_base_url: str = Field(default=None)
    openai_api_key: str = Field(default='')
    openai_model: str = Field(default='deepseek-ai/DeepSeek-R1-Distill-Qwen-7B')
    
llm_config = get_plugin_config(Config)



class LLMParams(BaseModel):
    """模型基础参数配置"""
    api_key: str = llm_config.openai_api_key
    base_url: str = llm_config.openai_base_url
    model: str = llm_config.openai_model
    temperature: float = 0.7
    max_tokens: int = 2000
    frequency_penalty: float = 0.0
    presence_penalty: float = 0.0
        

class PromptInjectionConfig(BaseModel):
    """提示词注入配置"""
    time: Literal[0, 1, 2, 3] = 1
    weather: Literal[0, 1] = 0
    
    
class HistoryConfig(BaseModel):
    """ History 配置"""
    db_path: str = str(DB_PATH)
    max_length: int = 10
    time_window: int = Field(86400, description="历史记录时间窗口(s)")
    


class SystemPromptConfig(BaseModel):
    """系统提示配置"""
    template: str = Field(
        "你是一个由openai公司开发的大模型gpt",
        description="基础提示词模板"
    )
    class Config:
        extra = 'allow'
    

class MessagesConfig(BaseModel):
    
    history: HistoryConfig = Field(default_factory=HistoryConfig)
    system_prompt: SystemPromptConfig = Field(default_factory=SystemPromptConfig)
    injections: PromptInjectionConfig = Field(default_factory=PromptInjectionConfig)
    
    
class AppConfig(BaseModel):
    """聚合应用配置"""
    params: LLMParams = Field(default_factory=LLMParams)
    messages: MessagesConfig = Field(default_factory=MessagesConfig)


def deep_update_model(model: BaseModel, update_data: dict[str, Any]) -> BaseModel:
    update_dict = model.model_dump()
    for key, value in update_data.items():
        if isinstance(value, dict) and isinstance(update_dict.get(key), BaseModel):
            # 递归更新嵌套的 BaseModel
            update_dict[key] = deep_update_model(update_dict[key], value)
        else:
            update_dict[key] = value
    return model.__class__(**update_dict)


from .interface import BaseLLMProvider
from ..config import LLMParams
from openai import APIError, APITimeoutError, AsyncOpenAI, AuthenticationError
from nonebot import  logger
from typing import Any, List, Tuple


class OpenAIProvider(BaseLLMProvider):
    def __init__(self):
        ...
      
    async def generate(self, messages: List[Tuple[str, str]], params: LLMParams) -> str:
        
        self.client = AsyncOpenAI(
            api_key = params.api_key, 
            base_url = params.base_url
        )
        
        try:
            response = await self.client.chat.completions.create(
                messages=messages,
                model=params.model,
                max_tokens=params.max_tokens,
                frequency_penalty=params.frequency_penalty,
                presence_penalty=params.presence_penalty,
                stream=False
            )
            return response.choices[0].message.content
        
        except APITimeoutError as e:
            logger.error(f"API 请求超时: {e}")
            raise "⚠️请求超时，请重试"
        except APIError as e:
            logger.error(f"API 错误: {e.status_code} - {e.message}")
            raise "⚠️服务暂时不可用"
        except AuthenticationError:
            logger.critical("API 密钥错误")
            raise "⚠️服务配置错误"
        except Exception as e:
            logger.error(f"未处理异常: {e}")
            raise "⚠️服务内部错误"

openai_provider = OpenAIProvider()